@@ -24,6 +24,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QApplication>
 #include <QDateTime>
 #include <QDesktopWidget>
+#include <QLabel>
 #include <QTimer>
 #include <QShortcut>
 #include <QSplitter>
@@ -72,6 +73,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "kipiinterface.h"
 #endif
 #ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
+#ifdef GWENVIEW_SEMANTICINFO_BACKEND_NEPOMUK
+#include <nepomuk/resourcemanager.h>
+#endif
 #include "semanticinfocontextmanageritem.h"
 #endif
 #include "preloader.h"
@@ -87,6 +91,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <lib/eventwatcher.h>
 #include <lib/fullscreenbar.h>
 #include <lib/gwenviewconfig.h>
+#include <lib/messagebubble.h>
 #include <lib/mimetypeutils.h>
 #include <lib/print/printhelper.h>
 #include <lib/slideshow.h>
@@ -97,6 +102,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <lib/thumbnailview/thumbnailview.h>
 #include <lib/thumbnailloadjob.h>
 #include <lib/urlutils.h>
+#include <lib/widgetfloater.h>
 
 namespace Gwenview {
 
@@ -117,6 +123,9 @@ static const char* FULLSCREEN_MODE_SIDE_BAR_GROUP = "SideBar-FullScreenMode";
 static const char* SIDE_BAR_IS_VISIBLE_KEY = "IsVisible";
 static const char* SIDE_BAR_CURRENT_PAGE_KEY = "CurrentPage";
 
+static const char* SESSION_CURRENT_PAGE_KEY = "Page";
+static const char* SESSION_URL_KEY = "Url";
+
 enum PageId {
 	StartPageId,
 	BrowsePageId,
@@ -180,22 +189,25 @@ struct MainWindow::Private {
 	KNotificationRestrictions* mNotificationRestrictions;
 
 	void setupWidgets() {
-		QWidget* centralWidget = new QWidget(mWindow);
-		mWindow->setCentralWidget(centralWidget);
-		mSaveBar = new SaveBar(centralWidget, mWindow->actionCollection());
-
-		mCentralSplitter = new Splitter(Qt::Horizontal, centralWidget);
-		QVBoxLayout* layout = new QVBoxLayout(centralWidget);
-		layout->addWidget(mSaveBar);
-		layout->addWidget(mCentralSplitter);
-		layout->setMargin(0);
-		layout->setSpacing(0);
+		mCentralSplitter = new Splitter(Qt::Horizontal, mWindow);
+		mWindow->setCentralWidget(mCentralSplitter);
 
+		// Left side of splitter
 		mSideBar = new SideBar(mCentralSplitter);
 		EventWatcher::install(mSideBar, QList<QEvent::Type>() << QEvent::Show << QEvent::Hide,
 			mWindow, SLOT(updateToggleSideBarAction()));
 
-		mViewStackedWidget = new QStackedWidget(mCentralSplitter);
+		// Right side of splitter
+		QWidget* contentWidget = new QWidget(mCentralSplitter);
+
+		mSaveBar = new SaveBar(contentWidget, mWindow->actionCollection());
+		mViewStackedWidget = new QStackedWidget(contentWidget);
+		QVBoxLayout* layout = new QVBoxLayout(contentWidget);
+		layout->addWidget(mSaveBar);
+		layout->addWidget(mViewStackedWidget);
+		layout->setMargin(0);
+		layout->setSpacing(0);
+		////
 
 		mSideBarCollapser = new SplitterCollapser(mCentralSplitter, mSideBar);
 
@@ -238,6 +250,8 @@ struct MainWindow::Private {
 		// Connect thumbnail view
 		connect(mThumbnailView, SIGNAL(indexActivated(const QModelIndex&)),
 			mWindow, SLOT(slotThumbnailViewIndexActivated(const QModelIndex&)) );
+		connect(mThumbnailView->model(), SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
+			mWindow, SLOT(slotDataChanged(const QModelIndex&, const QModelIndex&)) );
 		connect(mThumbnailView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)),
 			mWindow, SLOT(slotSelectionChanged()) );
 
@@ -409,9 +423,20 @@ struct MainWindow::Private {
 		mContextManager->addItem(infoItem);
 
 		#ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
-		SemanticInfoContextManagerItem* semanticInfoItem =
-			new SemanticInfoContextManagerItem(mContextManager, actionCollection, mDocumentPanel->documentView());
-		mContextManager->addItem(semanticInfoItem);
+		SemanticInfoContextManagerItem* semanticInfoItem = 0;
+		// When using Nepomuk, create a SemanticInfoContextManagerItem instance
+		// only if Nepomuk is available
+		// When using Fake backend always create it
+		#ifdef GWENVIEW_SEMANTICINFO_BACKEND_NEPOMUK
+		if (Nepomuk::ResourceManager::instance()->init() == 0) {
+		#endif
+			semanticInfoItem = new SemanticInfoContextManagerItem(mContextManager, actionCollection, mDocumentPanel->documentView());
+		#ifdef GWENVIEW_SEMANTICINFO_BACKEND_NEPOMUK
+		}
+		#endif
+		if (semanticInfoItem) {
+			mContextManager->addItem(semanticInfoItem);
+		}
 		#endif
 
 		ImageOpsContextManagerItem* imageOpsItem =
@@ -433,7 +458,9 @@ struct MainWindow::Private {
 		page->setObjectName("information");
 		page->addWidget(infoItem->widget());
 		#ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
-		page->addWidget(semanticInfoItem->widget());
+		if (semanticInfoItem) {
+			page->addWidget(semanticInfoItem->widget());
+		}
 		#endif
 		page->addStretch();
 		mSideBar->addPage(page);
@@ -473,29 +500,24 @@ struct MainWindow::Private {
 		mDirModel->setKindFilter(kinds);
 	}
 
-	QModelIndex getRelativeIndex(int offset) {
+	QModelIndex currentIndex() const {
 		KUrl url = currentUrl();
-		if (!url.isValid()) {
-			return QModelIndex();
-		}
-		QModelIndex index = mDirModel->indexForUrl(url);
-		int row = index.row() + offset;
-		index = mDirModel->index(row, 0);
-		if (!index.isValid()) {
-			return QModelIndex();
-		}
+		return url.isValid() ? mDirModel->indexForUrl(url) : QModelIndex();
+	}
 
+	bool indexIsDirOrArchive(const QModelIndex& index) const {
+		Q_ASSERT(index.isValid());
 		KFileItem item = mDirModel->itemForIndex(index);
-		if (!ArchiveUtils::fileItemIsDirOrArchive(item)) {
-			return index;
-		} else {
-			return QModelIndex();
-		}
+		return ArchiveUtils::fileItemIsDirOrArchive(item);
 	}
 
 	void goTo(int offset) {
-		QModelIndex index = getRelativeIndex(offset);
-		if (index.isValid()) {
+		QModelIndex index = currentIndex();
+		index = mDirModel->index(index.row() + offset, 0);
+		if (!index.isValid()) {
+			return;
+		}
+		if (!indexIsDirOrArchive(index)) {
 			mThumbnailView->setCurrentIndex(index);
 			mThumbnailView->scrollTo(index);
 		}
@@ -509,8 +531,7 @@ struct MainWindow::Private {
 				return;
 			}
 
-			KFileItem item = mDirModel->itemForIndex(index);
-			if (!ArchiveUtils::fileItemIsDirOrArchive(item)) {
+			if (!indexIsDirOrArchive(index)) {
 				break;
 			}
 		}
@@ -988,9 +1009,6 @@ void MainWindow::openDirUrl(const KUrl& url) {
 
 
 void MainWindow::openDocumentUrl(const KUrl& url) {
-	if (d->mDocumentPanel->url() == url) {
-		return;
-	}
 	d->mDocumentPanel->openUrl(url);
 
 	d->mFullScreenContent->setCurrentUrl(url);
@@ -1071,16 +1089,40 @@ void MainWindow::slotSelectionChanged() {
 }
 
 
-void MainWindow::slotDirModelNewItems() {
-	QItemSelection selection = d->mThumbnailView->selectionModel()->selection();
-	if (selection.size() > 0) {
-		updatePreviousNextActions();
+void MainWindow::slotDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight) {
+	QModelIndexList selectionList = d->mThumbnailView->selectionModel()->selectedIndexes();
+	if (selectionList.isEmpty()) {
 		return;
 	}
 
-	// Nothing selected in the view yet, check if there was an url waiting to
-	// be selected
-	d->selectUrlToSelect();
+	QModelIndexList changedList;
+	const QAbstractItemModel* model = topLeft.model();
+	for (int row=topLeft.row(); row <= bottomRight.row(); ++row) {
+		changedList << model->index(row, 0);
+	}
+
+	QModelIndexList& shortList = selectionList;
+	QModelIndexList& longList = changedList;
+	if (shortList.length() > longList.length()) {
+		qSwap(shortList, longList);
+	}
+	Q_FOREACH(const QModelIndex& index, shortList) {
+		if (longList.contains(index)) {
+			d->updateContextDependentComponents();
+			return;
+		}
+	}
+}
+
+
+void MainWindow::slotDirModelNewItems() {
+	if (d->mThumbnailView->selectionModel()->hasSelection()) {
+		updatePreviousNextActions();
+	} else {
+		// Nothing selected in the view yet, check if there was an url waiting to
+		// be selected
+		d->selectUrlToSelect();
+	}
 }
 
 
@@ -1091,7 +1133,7 @@ void MainWindow::slotDirListerCompleted() {
 	}
 	if (d->mThumbnailView->selectionModel()->hasSelection()) {
 		updatePreviousNextActions();
-	} else {
+	} else if (!d->mUrlToSelect.isValid()) {
 		d->goToFirstDocument();
 	}
 }
@@ -1108,6 +1150,7 @@ void MainWindow::goToNext() {
 
 
 void MainWindow::goToUrl(const KUrl& url) {
+	kDebug() << url;
 	if (d->mCurrentPageId == ViewPageId) {
 		openDocumentUrl(url);
 	}
@@ -1123,13 +1166,13 @@ void MainWindow::goToUrl(const KUrl& url) {
 
 
 void MainWindow::updatePreviousNextActions() {
-	QModelIndex index;
+	QModelIndex index = d->currentIndex();
 
-	index = d->getRelativeIndex(-1);
-	d->mGoToPreviousAction->setEnabled(index.isValid());
+	QModelIndex prevIndex = d->mDirModel->index(index.row() - 1, 0);
+	d->mGoToPreviousAction->setEnabled(prevIndex.isValid() && !d->indexIsDirOrArchive(prevIndex));
 
-	index = d->getRelativeIndex(1);
-	d->mGoToNextAction->setEnabled(index.isValid());
+	QModelIndex nextIndex = d->mDirModel->index(index.row() + 1, 0);
+	d->mGoToNextAction->setEnabled(nextIndex.isValid() && !d->indexIsDirOrArchive(nextIndex));
 }
 
 
@@ -1465,4 +1508,37 @@ void MainWindow::setDistractionFreeMode(bool value) {
 }
 
 
+void MainWindow::showMessageBubble(MessageBubble* bubble) {
+	WidgetFloater* floater = new WidgetFloater(centralWidget());
+	floater->setChildWidget(bubble);
+	floater->setAlignment(Qt::AlignHCenter | Qt::AlignTop);
+	bubble->show();
+}
+
+
+void MainWindow::saveProperties(KConfigGroup& group) {
+	group.writeEntry(SESSION_CURRENT_PAGE_KEY, int(d->mCurrentPageId));
+	group.writeEntry(SESSION_URL_KEY, d->currentUrl());
+}
+
+
+void MainWindow::readProperties(const KConfigGroup& group) {
+	PageId pageId = PageId(group.readEntry(SESSION_CURRENT_PAGE_KEY, int(StartPageId)));
+	if (pageId == StartPageId) {
+		d->mCurrentPageId = StartPageId;
+		showStartPage();
+	} else if (pageId == BrowsePageId) {
+		d->mBrowseAction->trigger();
+	} else {
+		d->mViewAction->trigger();
+	}
+	KUrl url = group.readEntry(SESSION_URL_KEY, KUrl());
+	if (!url.isValid()) {
+		kWarning() << "Invalid url!";
+		return;
+	}
+	goToUrl(url);
+}
+
+
 } // namespace
