@@ -1,7 +1,7 @@
 // vim: set tabstop=4 shiftwidth=4 noexpandtab:
 /*
 Gwenview: an image viewer
-Copyright 2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2007 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,14 +24,16 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 // Qt
 
 // KDE
+#include <kurl.h>
 
 // Local
+#include "document/documentfactory.h"
 
 namespace Gwenview {
 
 
 struct AbstractImageOperationPrivate {
-	Document::Ptr mDocument;
+	KUrl mUrl;
 };
 
 
@@ -47,12 +49,15 @@ AbstractImageOperation::~AbstractImageOperation() {
 
 
 void AbstractImageOperation::setDocument(Document::Ptr doc) {
-	d->mDocument = doc;
+	d->mUrl = doc->url();
 }
 
 
 Document::Ptr AbstractImageOperation::document() const {
-	return d->mDocument;
+	Document::Ptr doc = DocumentFactory::instance()->load(d->mUrl);
+	doc->loadFullImage();
+	doc->waitUntilLoaded();
+	return doc;
 }
 
 
