@@ -1,5 +1,5 @@
 /***************************************************************************
-                          klineal.h  -  description
+                          krulersystemtray.cpp  -  description
                              -------------------
     Copyright            : (C) 2009        by Montel Laurent <montel@kde.org>
  ***************************************************************************/
@@ -14,14 +14,16 @@
  ***************************************************************************/
 
 #include "krulersystemtray.h"
-#include <QMenu>
 
+#include <KLocale>
+#include <KMenu>
 
-KRulerSystemTray::KRulerSystemTray( const KIcon& icon, QWidget * parent, KActionCollection *actions)
-    : KSystemTrayIcon( icon, parent )
+KRulerSystemTray::KRulerSystemTray( const QString& iconName, QWidget *parent, KActionCollection *actions)
+  : KStatusNotifierItem( parent )
 {
-    QMenu * cm = contextMenu();
-    cm->addAction( actions->action("preferences") );
-
+  setIconByName( iconName );
+  setStatus(KStatusNotifierItem::Active);
+  setToolTip( iconName, i18n( "KDE Screen Ruler" ), QString() );
+  KMenu *cm = contextMenu();
+  cm->addAction( actions->action( "preferences" ) );
 }
-
