@@ -22,6 +22,10 @@
  *
  * ============================================================ */
 
+// KDE includes
+
+#include <klocale.h>
+
 // Local includes.
 
 #include "kexiv2_p.h"
@@ -43,7 +47,7 @@ bool KExiv2::canWriteExif(const QString& filePath)
     catch( Exiv2::Error &e )
     {
         std::string s(e.what());
-        kDebug(51003) << "Cannot check Exif access mode using Exiv2 (Error #" 
+        kDebug(51003) << "Cannot check Exif access mode using Exiv2 (Error #"
                       << e.code() << ": " << s.c_str() << ")" << endl;
     }
 
@@ -52,14 +56,14 @@ bool KExiv2::canWriteExif(const QString& filePath)
 
 bool KExiv2::hasExif() const
 {
-    return !d->exifMetadata.empty();
+    return !d->exifMetadata().empty();
 }
 
 bool KExiv2::clearExif() const
 {
     try
     {
-        d->exifMetadata.clear();
+        d->exifMetadata().clear();
         return true;
     }
     catch( Exiv2::Error &e )
@@ -71,13 +75,18 @@ bool KExiv2::clearExif() const
 }
 
 QByteArray KExiv2::getExif(bool addExifHeader) const
+{
+    return getExifEncoded(addExifHeader);
+}
+
+QByteArray KExiv2::getExifEncoded(bool addExifHeader) const
 {
     try
     {
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
             QByteArray data;
-            Exiv2::ExifData& exif = d->exifMetadata;
+            Exiv2::ExifData& exif = d->exifMetadata();
 #if (EXIV2_TEST_VERSION(0,17,91))
             Exiv2::Blob blob;
             Exiv2::ExifParser::encode(blob, Exiv2::bigEndian, exif);
@@ -118,10 +127,10 @@ bool KExiv2::setExif(const QByteArray& data) const
         if (!data.isEmpty())
         {
 #if (EXIV2_TEST_VERSION(0,17,91))
-            Exiv2::ExifParser::decode(d->exifMetadata, (const Exiv2::byte*)data.data(), data.size());
-            return (!d->exifMetadata.empty());
+            Exiv2::ExifParser::decode(d->exifMetadata(), (const Exiv2::byte*)data.data(), data.size());
+            return (!d->exifMetadata().empty());
 #else
-            if (d->exifMetadata.load((const Exiv2::byte*)data.data(), data.size()) != 0)
+            if (d->exifMetadata().load((const Exiv2::byte*)data.data(), data.size()) != 0)
                 return false;
             else
                 return true;
@@ -141,12 +150,12 @@ bool KExiv2::setExif(const QByteArray& data) const
 
 KExiv2::MetaDataMap KExiv2::getExifTagsDataList(const QStringList &exifKeysFilter, bool invertSelection) const
 {
-    if (d->exifMetadata.empty())
+    if (d->exifMetadata().empty())
        return MetaDataMap();
 
     try
     {
-        Exiv2::ExifData exifData = d->exifMetadata;
+        Exiv2::ExifData exifData = d->exifMetadata();
         exifData.sortByKey();
 
         QString     ifDItemName;
@@ -162,6 +171,10 @@ KExiv2::MetaDataMap KExiv2::getExifTagsDataList(const QStringList &exifKeysFilte
             {
                 tagValue = d->convertCommentValue(*md);
             }
+            else if (key == "Exif.Image.0x935c")
+            {
+                tagValue = i18n("Data of size %1", md->value().size());
+            }
             else
             {
                 std::ostringstream os;
@@ -200,10 +213,10 @@ QString KExiv2::getExifComment() const
 {
     try
     {
-        if (!d->exifMetadata.empty())
+        if (!d->exifMetadata().empty())
         {
             Exiv2::ExifKey key("Exif.Photo.UserComment");
-            Exiv2::ExifData exifData(d->exifMetadata);
+            Exiv2::ExifData exifData(d->exifMetadata());
             Exiv2::ExifData::iterator it = exifData.findKey(key);
 
             if (it != exifData.end())
@@ -242,7 +255,7 @@ bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
                 // write as ASCII
                 std::string exifComment("charset=\"Ascii\" ");
                 exifComment += comment.toLatin1().constData();
-                d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
+                d->exifMetadata()["Exif.Photo.UserComment"] = exifComment;
             }
             else
             {
@@ -255,7 +268,7 @@ bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
                 const unsigned short *ucs2 = comment.utf16();
                 std::string exifComment("charset=\"Unicode\" ");
                 exifComment.append((const char*)ucs2, sizeof(unsigned short) * comment.length());
-                d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
+                d->exifMetadata()["Exif.Photo.UserComment"] = exifComment;
             }
         }
         return true;
@@ -270,13 +283,13 @@ bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
 
 QString KExiv2::getExifTagTitle(const char *exifTagName)
 {
-    try 
+    try
     {
         std::string exifkey(exifTagName);
-        Exiv2::ExifKey ek(exifkey); 
+        Exiv2::ExifKey ek(exifkey);
         return QString::fromLocal8Bit( Exiv2::ExifTags::tagTitle(ek.tag(), ek.ifdId()) );
     }
-    catch (Exiv2::Error& e) 
+    catch (Exiv2::Error& e)
     {
         d->printExiv2ExceptionError("Cannot get metadata tag title using Exiv2 ", e);
     }
@@ -286,13 +299,13 @@ QString KExiv2::getExifTagTitle(const char *exifTagName)
 
 QString KExiv2::getExifTagDescription(const char *exifTagName)
 {
-    try 
+    try
     {
         std::string exifkey(exifTagName);
-        Exiv2::ExifKey ek(exifkey); 
+        Exiv2::ExifKey ek(exifkey);
         return QString::fromLocal8Bit( Exiv2::ExifTags::tagDesc(ek.tag(), ek.ifdId()) );
     }
-    catch (Exiv2::Error& e) 
+    catch (Exiv2::Error& e)
     {
         d->printExiv2ExceptionError("Cannot get metadata tag description using Exiv2 ", e);
     }
@@ -308,10 +321,10 @@ bool KExiv2::removeExifTag(const char *exifTagName, bool setProgramName) const
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData::iterator it = d->exifMetadata.findKey(exifKey);
-        if (it != d->exifMetadata.end())
+        Exiv2::ExifData::iterator it = d->exifMetadata().findKey(exifKey);
+        if (it != d->exifMetadata().end())
         {
-            d->exifMetadata.erase(it);
+            d->exifMetadata().erase(it);
             return true;
         }
     }
@@ -328,7 +341,7 @@ bool KExiv2::getExifTagRational(const char *exifTagName, long int &num, long int
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -339,7 +352,7 @@ bool KExiv2::getExifTagRational(const char *exifTagName, long int &num, long int
     }
     catch( Exiv2::Error &e )
     {
-        d->printExiv2ExceptionError(QString("Cannot find Exif Rational value from key '%1' " 
+        d->printExiv2ExceptionError(QString("Cannot find Exif Rational value from key '%1' "
                                          "into image using Exiv2 ").arg(exifTagName), e);
     }
 
@@ -353,7 +366,7 @@ bool KExiv2::setExifTagLong(const char *exifTagName, long val, bool setProgramNa
 
     try
     {
-        d->exifMetadata[exifTagName] = static_cast<int32_t>(val);
+        d->exifMetadata()[exifTagName] = static_cast<int32_t>(val);
         return true;
     }
     catch( Exiv2::Error &e )
@@ -371,7 +384,7 @@ bool KExiv2::setExifTagRational(const char *exifTagName, long int num, long int
 
     try
     {
-        d->exifMetadata[exifTagName] = Exiv2::Rational(num, den);
+        d->exifMetadata()[exifTagName] = Exiv2::Rational(num, den);
         return true;
     }
     catch( Exiv2::Error &e )
@@ -393,7 +406,7 @@ bool KExiv2::setExifTagData(const char *exifTagName, const QByteArray& data, boo
     try
     {
         Exiv2::DataValue val((Exiv2::byte *)data.data(), data.size());
-        d->exifMetadata[exifTagName] = val;
+        d->exifMetadata()[exifTagName] = val;
         return true;
     }
     catch( Exiv2::Error &e )
@@ -404,7 +417,7 @@ bool KExiv2::setExifTagData(const char *exifTagName, const QByteArray& data, boo
     return false;
 }
 
-bool KExiv2::setExifTagVariant(const char *exifTagName, const QVariant& val, 
+bool KExiv2::setExifTagVariant(const char *exifTagName, const QVariant& val,
                                bool rationalWantSmallDenominator, bool setProgramName) const
 {
     switch (val.type())
@@ -449,7 +462,7 @@ bool KExiv2::setExifTagVariant(const char *exifTagName, const QVariant& val,
             try
             {
                 const std::string &exifdatetime(dateTime.toString(QString("yyyy:MM:dd hh:mm:ss")).toAscii().constData());
-                d->exifMetadata[exifTagName] = exifdatetime;
+                d->exifMetadata()[exifTagName] = exifdatetime;
             }
             catch( Exiv2::Error &e )
             {
@@ -555,7 +568,7 @@ bool KExiv2::getExifTagLong(const char* exifTagName, long &val) const
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end() && it->count() > 0)
         {
@@ -577,7 +590,7 @@ QByteArray KExiv2::getExifTagData(const char* exifTagName) const
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -588,7 +601,7 @@ QByteArray KExiv2::getExifTagData(const char* exifTagName) const
             (*it).copy((Exiv2::byte*)s, exifData.byteOrder());
 #endif
             QByteArray data(s, (*it).size());
-            delete s;
+            delete[] s;
             return data;
         }
     }
@@ -606,7 +619,7 @@ QVariant KExiv2::getExifTagVariant(const char *exifTagName, bool rationalAsListO
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -681,7 +694,7 @@ QString KExiv2::getExifTagString(const char* exifTagName, bool escapeCR) const
     try
     {
         Exiv2::ExifKey exifKey(exifTagName);
-        Exiv2::ExifData exifData(d->exifMetadata);
+        Exiv2::ExifData exifData(d->exifMetadata());
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
@@ -716,7 +729,7 @@ bool KExiv2::setExifTagString(const char *exifTagName, const QString& value, boo
 
     try
     {
-        d->exifMetadata[exifTagName] = std::string(value.toAscii().constData());
+        d->exifMetadata()[exifTagName] = std::string(value.toAscii().constData());
         return true;
     }
     catch( Exiv2::Error &e )
@@ -731,16 +744,16 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
 {
     QImage thumbnail;
 
-    if (d->exifMetadata.empty())
+    if (d->exifMetadata().empty())
        return thumbnail;
 
     try
     {
 #if (EXIV2_TEST_VERSION(0,17,91))
-        Exiv2::ExifThumbC thumb(d->exifMetadata);
+        Exiv2::ExifThumbC thumb(d->exifMetadata());
         Exiv2::DataBuf const c1 = thumb.copy();
 #else
-        Exiv2::DataBuf const c1(d->exifMetadata.copyThumbnail());
+        Exiv2::DataBuf const c1(d->exifMetadata().copyThumbnail());
 #endif
         thumbnail.loadFromData(c1.pData_, c1.size_);
 
@@ -748,9 +761,12 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
         {
             if (fixOrientation)
             {
-                Exiv2::ExifKey key("Exif.Thumbnail.Orientation");
-                Exiv2::ExifData exifData(d->exifMetadata);
-                Exiv2::ExifData::iterator it = exifData.findKey(key);
+                Exiv2::ExifKey key1("Exif.Thumbnail.Orientation");
+                Exiv2::ExifKey key2("Exif.Image.Orientation");
+                Exiv2::ExifData exifData(d->exifMetadata());
+                Exiv2::ExifData::iterator it = exifData.findKey(key1);
+                if (it == exifData.end())
+                    it = exifData.findKey(key2);
                 if (it != exifData.end())
                 {
                     long orientation = it->toLong();
@@ -831,7 +847,7 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
         QBuffer buffer(&data);
         buffer.open(QIODevice::WriteOnly);
         thumb.save(&buffer, "JPEG");
-        Exiv2::ExifThumb thumb(d->exifMetadata);
+        Exiv2::ExifThumb thumb(d->exifMetadata());
         thumb.setJpegThumbnail((Exiv2::byte *)data.data(), data.size());
 #else
         KTemporaryFile thumbFile;
@@ -841,7 +857,7 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
         thumb.save(thumbFile.fileName(), "JPEG");
         kDebug(51003) << "Thumbnail temp file: " << thumbFile.fileName().toAscii().data() << endl;
         const std::string &fileName((const char*)(QFile::encodeName(thumbFile.fileName())));
-        d->exifMetadata.setJpegThumbnail( fileName );
+        d->exifMetadata().setJpegThumbnail( fileName );
 #endif
         return true;
     }
@@ -853,4 +869,115 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
     return false;
 }
 
+KExiv2::TagsMap KExiv2::getStdExifTagsList() const
+{
+    try
+    {
+        QList<const Exiv2::TagInfo*> tags;
+        tags << Exiv2::ExifTags::ifdTagList()
+             << Exiv2::ExifTags::exifTagList()
+             << Exiv2::ExifTags::iopTagList()
+             << Exiv2::ExifTags::gpsTagList();
+
+        TagsMap tagsMap;
+        for (QList<const Exiv2::TagInfo*>::iterator it = tags.begin(); it != tags.end(); ++it)
+        {
+            do
+            {
+                QString     key = QLatin1String( Exiv2::ExifKey( (*it)->tag_, Exiv2::ExifTags::ifdItem( (*it)->ifdId_ ) ).key().c_str() );
+                QStringList values;
+                values << (*it)->name_ << (*it)->title_ << (*it)->desc_;
+                tagsMap.insert(key, values);
+                ++(*it);
+            }
+            while((*it)->tag_ != 0xffff);
+        }
+        return tagsMap;
+    }
+    catch( Exiv2::Error &e )
+    {
+        d->printExiv2ExceptionError("Cannot get Exif Tags list using Exiv2 ", e);
+    }
+
+    return TagsMap();
+}
+
+KExiv2::TagsMap KExiv2::getMakernoteTagsList() const
+{
+#if (EXIV2_TEST_VERSION(0,18,1))
+    try
+    {
+        QList<const Exiv2::TagInfo*> tags;
+        tags
+             // Canon Makernotes.
+             << Exiv2::CanonMakerNote::tagList()
+             << Exiv2::CanonMakerNote::tagListCs()
+             << Exiv2::CanonMakerNote::tagListSi()
+             << Exiv2::CanonMakerNote::tagListPa()
+             << Exiv2::CanonMakerNote::tagListCf()
+             << Exiv2::CanonMakerNote::tagListPi()
+
+             /*
+              * Not yet officially released. I added some new tags to the
+              * Canon Makernote namespace. Should be released with exiv2-0.19...
+              *
+                     << Exiv2::CanonMakerNote::tagListFi()
+              */
+
+             // Sigma Makernotes.
+             << Exiv2::SigmaMakerNote::tagList()
+             // Sony Makernotes.
+             << Exiv2::SonyMakerNote::tagList()
+             // Minolta Makernotes.
+             << Exiv2::MinoltaMakerNote::tagList()
+             << Exiv2::MinoltaMakerNote::tagListCsStd()
+             << Exiv2::MinoltaMakerNote::tagListCs7D()
+             << Exiv2::MinoltaMakerNote::tagListCs5D()
+             // Nikon Makernotes.
+             << Exiv2::Nikon1MakerNote::tagList()
+             << Exiv2::Nikon2MakerNote::tagList()
+             << Exiv2::Nikon3MakerNote::tagList()
+             // Olympus Makernotes.
+             << Exiv2::OlympusMakerNote::tagList()
+             << Exiv2::OlympusMakerNote::tagListCs()
+             << Exiv2::OlympusMakerNote::tagListEq()
+             << Exiv2::OlympusMakerNote::tagListRd()
+             << Exiv2::OlympusMakerNote::tagListRd2()
+             << Exiv2::OlympusMakerNote::tagListIp()
+             << Exiv2::OlympusMakerNote::tagListFi()
+             << Exiv2::OlympusMakerNote::tagListFe()
+             << Exiv2::OlympusMakerNote::tagList()
+             << Exiv2::OlympusMakerNote::tagListRi()
+             // Panasonic Makernotes.
+             << Exiv2::PanasonicMakerNote::tagList()
+             << Exiv2::PanasonicMakerNote::tagListRaw()
+             // Pentax Makernotes.
+             << Exiv2::PentaxMakerNote::tagList()
+             // Fuji Makernotes.
+             << Exiv2::FujiMakerNote::tagList();
+
+        TagsMap tagsMap;
+        for (QList<const Exiv2::TagInfo*>::iterator it = tags.begin(); it != tags.end(); ++it)
+        {
+            do
+            {
+                QString     key = QLatin1String( Exiv2::ExifKey( (*it)->tag_, Exiv2::ExifTags::ifdItem( (*it)->ifdId_ ) ).key().c_str() );
+                QStringList values;
+                values << (*it)->name_ << (*it)->title_ << (*it)->desc_;
+                tagsMap.insert(key, values);
+                ++(*it);
+            }
+            while((*it)->tag_ != 0xffff);
+        }
+        return tagsMap;
+    }
+    catch( Exiv2::Error &e )
+    {
+        d->printExiv2ExceptionError("Cannot get Makernote Tags list using Exiv2 ", e);
+    }
+#endif
+
+    return TagsMap();
+}
+
 }  // NameSpace KExiv2Iface
